@@ -116,11 +116,12 @@ imputeX <- function(model, X, weight = NULL, U = NULL, U_bin = NULL) {
                       model$matrices$B,
                       model$matrices$Bi, model$info$add_implicit_features,
                       model$info$k, model$info$k_user, model$info$k_item, model$info$k_main,
-                      model$info$lambda,
+                      model$info$lambda, model$info$l1_lambda,
+                      model$info$scale_lam, model$info$scale_lam_sideinfo,
                       model$info$w_main, model$info$w_user, model$info$w_implicit,
                       NCOL(model$matrices$B), model$info$include_all_X,
-                      model$precomputed$TransBtBinvBt,
                       model$precomputed$BtB,
+                      model$precomputed$TransBtBinvBt,
                       model$precomputed$BeTBeChol,
                       model$precomputed$BiTBi,
                       model$precomputed$TransCtCinvCt,
