@@ -80,6 +80,7 @@ swap.users.and.items <- function(model, precompute = TRUE) {
             k_main = model$info$k_main,
             k_sec = model$info$k_sec,
             lambda = swap.lambda(model$info$lambda),
+            l1_lambda = swap.lambda(model$info$l1_lambda),
             alpha = model$info$alpha,
             nfev = model$info$nfev,
             nupd = model$info$nupd,
@@ -97,6 +98,9 @@ swap.users.and.items <- function(model, precompute = TRUE) {
             nonneg = model$info$nonneg,
             add_implicit_features = model$info$add_implicit_features,
             include_all_X = model$info$include_all_X,
+            center = model$info$center,
+            scale_lam = model$info$scale_lam,
+            scale_lam_sideinfo = model$info$scale_lam_sideinfo,
             nthreads = model$info$nthreads
         ),
         matrices = list(
@@ -140,11 +144,13 @@ swap.users.and.items <- function(model, precompute = TRUE) {
                               new_model$matrices$Bm, n_max, n_max, TRUE,
                               numeric(), 0L,
                               numeric(), FALSE,
+                              numeric(), numeric(1L), FALSE,
                               new_model$info$k_sec + new_model$info$k + new_model$info$k_main,
                               0L, 0L, 0L,
                               as.logical(NROW(new_model$matries$user_bias)),
                               FALSE,
                               new_model$info$lambda,
+                              FALSE, FALSE,
                               1., 1., 1.,
                               new_model$precomputed$Bm_plus_bias,
                               new_model$precomputed$BtB,
@@ -152,6 +158,7 @@ swap.users.and.items <- function(model, precompute = TRUE) {
                               numeric(),
                               numeric(),
                               numeric(),
+                              numeric(),
                               numeric())
             check.ret.code(ret_code)
         } else if ("OMF_implicit" %in% class(model)) {
