@@ -149,11 +149,13 @@ predict_new.CMF <- function(model, items, rows=NULL,
                       model$matrices$B,
                       model$matrices$Bi, model$info$add_implicit_features,
                       model$info$k, model$info$k_user, model$info$k_item, model$info$k_main,
-                      model$info$lambda,
+                      model$info$lambda, model$info$l1_lambda,
+                      model$info$scale_lam, model$info$scale_lam_sideinfo,
                       model$info$w_main, model$info$w_user, model$info$w_implicit,
                       NCOL(model$matrices$B), model$info$include_all_X,
-                      model$precomputed$TransBtBinvBt,
                       model$precomputed$BtB,
+                      model$precomputed$TransBtBinvBt,
+                      model$precomputed$BtXbias,
                       model$precomputed$BeTBeChol,
                       model$precomputed$BiTBi,
                       model$precomputed$TransCtCinvCt,
@@ -173,6 +175,7 @@ predict_new.CMF_implicit <- function(model, items, rows=NULL,
     n_predict <- NROW(inputs$rows)
     scores <- numeric(length = n_predict)
     lambda <- ifelse(NROW(model$info$lambda) == 6L, model$info$lambda[3L], model$info$lambda)
+    l1_lambda <- ifelse(NROW(model$info$l1_lambda) == 6L, model$info$l1_lambda[3L], model$info$l1_lambda)
     
     ret_code <- .Call("call_predict_X_new_collective_implicit",
                       inputs$m_max,
@@ -189,7 +192,7 @@ predict_new.CMF_implicit <- function(model, items, rows=NULL,
                       model$matrices$C,
                       model$matrices$U_colmeans,
                       model$info$k, model$info$k_user, model$info$k_item, model$info$k_main,
-                      lambda, model$info$alpha, model$info$w_main, model$info$w_user,
+                      lambda, l1_lambda, model$info$alpha, model$info$w_main, model$info$w_user,
                       model$info$w_main_multiplier,
                       model$info$apply_log_transf,
                       model$precomputed$BeTBe,
