@@ -32,6 +32,10 @@
             non-negative least squares problem."
             International Conference on Computer Analysis of Images
             and Patterns. Springer, Berlin, Heidelberg, 2005.
+        (e) Zhou, Yunhong, et al.
+            "Large-scale parallel collaborative filtering for the netflix prize."
+            International conference on algorithmic applications in management.
+            Springer, Berlin, Heidelberg, 2008.
 
     For information about the models offered here and how they are fit to
     the data, see the files 'collective.c' and 'offsets.c'.
@@ -422,6 +426,8 @@ void print_oom_message(void);
 #ifdef _FOR_R
 void R_nan_to_C_nan(real_t arr[], size_t n);
 #endif
+long double compensated_sum(real_t *arr, size_t n);
+long double compensated_sum_product(real_t *restrict arr1, real_t *restrict arr2, size_t n);
 
 
 /* common.c */
@@ -452,12 +458,15 @@ void factors_closed_form
     real_t *restrict weight,
     real_t *restrict buffer_real_t,
     real_t lam, real_t lam_last,
+    real_t l1_lam, real_t l1_lam_last,
+    bool scale_lam,
     real_t *restrict precomputedTransBtBinvBt,
     real_t *restrict precomputedBtB, int_t cnt_NA, int_t ld_BtB,
     bool BtB_has_diag, bool BtB_is_scaled, real_t scale_BtB, int_t n_BtB,
     real_t *restrict precomputedBtBchol, bool NA_as_zero,
     bool use_cg, int_t max_cg_steps,/* <- 'cg' should not be used for new data*/
-    bool nonneg, int_t max_cd_steps, real_t *restrict a_prev,
+    bool nonneg, int_t max_cd_steps,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     bool force_add_diag
 );
 void factors_explicit_cg
@@ -477,6 +486,7 @@ void factors_explicit_cg_NA_as_zero_weighted
     real_t *restrict Xa, int_t ixB[], size_t nnz,
     real_t *restrict weight,
     real_t *restrict precomputedBtB, int_t ld_BtB,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     real_t *restrict buffer_real_t,
     real_t lam, real_t lam_last,
     int_t max_cg_steps
@@ -507,17 +517,18 @@ void factors_implicit_chol
     real_t *restrict a_vec, int_t k,
     real_t *restrict B, size_t ldb,
     real_t *restrict Xa, int_t ixB[], size_t nnz,
-    real_t lam,
+    real_t lam, real_t l1_lam,
     real_t *restrict precomputedBtB, int_t ld_BtB,
-    bool nonneg, int_t max_cd_steps, real_t *restrict a_prev,
+    bool nonneg, int_t max_cd_steps,
     real_t *restrict buffer_real_t
 );
 void solve_nonneg
 (
     real_t *restrict BtB,
     real_t *restrict BtX, /* <- solution will be here */
-    real_t *restrict a_prev,
+    real_t *restrict buffer_real_t,
     int_t k,
+    real_t l1_lam, real_t l1_lam_last,
     size_t max_cd_steps,
     bool fill_lower
 );
@@ -525,8 +536,29 @@ void solve_nonneg_batch
 (
     real_t *restrict BtB,
     real_t *restrict BtX, /* <- solution will be here */
-    real_t *restrict A_prev,
+    real_t *restrict buffer_real_t,
     int_t m, int_t k, size_t lda,
+    real_t l1_lam, real_t l1_lam_last,
+    size_t max_cd_steps,
+    int nthreads
+);
+void solve_elasticnet
+(
+    real_t *restrict BtB,
+    real_t *restrict BtX, /* <- solution will be here */
+    real_t *restrict buffer_real_t,
+    int_t k,
+    real_t l1_lam, real_t l1_lam_last,
+    size_t max_cd_steps,
+    bool fill_lower
+);
+void solve_elasticnet_batch
+(
+    real_t *restrict BtB,
+    real_t *restrict BtX, /* <- solution will be here */
+    real_t *restrict buffer_real_t,
+    int_t m, int_t k, size_t lda,
+    real_t l1_lam, real_t l1_lam_last,
     size_t max_cd_steps,
     int nthreads
 );
@@ -588,6 +620,7 @@ size_t buffer_size_optimizeA
 (
     size_t n, bool full_dense, bool near_dense, bool do_B,
     bool has_dense, bool has_weights, bool NA_as_zero,
+    bool nonneg, bool has_l1,
     size_t k, size_t nthreads,
     bool pass_allocated_BtB, bool keep_precomputedBtB,
     bool use_cg, bool finalize_chol
@@ -596,6 +629,7 @@ size_t buffer_size_optimizeA_implicit
 (
     size_t k, size_t nthreads,
     bool pass_allocated_BtB,
+    bool nonneg, bool has_l1,
     bool use_cg, bool finalize_chol
 );
 void optimizeA
@@ -607,10 +641,13 @@ void optimizeA
     real_t *restrict Xfull, int_t ldX, bool full_dense, bool near_dense,
     int_t cnt_NA[], real_t *restrict weight, bool NA_as_zero,
     real_t lam, real_t lam_last,
+    real_t l1_lam, real_t l1_lam_last,
+    bool scale_lam,
     bool do_B, bool is_first_iter,
     int_t nthreads,
     bool use_cg, int_t max_cg_steps,
-    bool nonneg, int_t max_cd_steps, real_t *restrict A_prev,
+    bool nonneg, int_t max_cd_steps,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     bool keep_precomputedBtB,
     real_t *restrict precomputedBtB, bool *filled_BtB,
     real_t *restrict buffer_real_t
@@ -621,18 +658,19 @@ void optimizeA_implicit
     real_t *restrict B, size_t ldb,
     int_t m, int_t n, int_t k,
     size_t Xcsr_p[], int_t Xcsr_i[], real_t *restrict Xcsr,
-    real_t lam,
+    real_t lam, real_t l1_lam,
     int_t nthreads,
     bool use_cg, int_t max_cg_steps, bool force_set_to_zero,
-    bool nonneg, int_t max_cd_steps, real_t *restrict A_prev,
+    bool nonneg, int_t max_cd_steps,
     real_t *restrict precomputedBtB, /* <- will be calculated if not passed */
     real_t *restrict buffer_real_t
 );
 int_t initialize_biases
 (
     real_t *restrict glob_mean, real_t *restrict biasA, real_t *restrict biasB,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     real_t lam_user, real_t lam_item,
+    bool scale_lam,
     int_t m, int_t n,
     int_t m_bias, int_t n_bias,
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
@@ -693,6 +731,23 @@ int_t fit_most_popular
     real_t *restrict biasA, real_t *restrict biasB,
     real_t *restrict glob_mean,
     real_t lam_user, real_t lam_item,
+    bool scale_lam,
+    real_t alpha,
+    int_t m, int_t n,
+    int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
+    real_t *restrict Xfull,
+    real_t *restrict weight,
+    bool implicit, bool adjust_weight, bool apply_log_transf,
+    bool nonneg,
+    real_t *restrict w_main_multiplier,
+    int_t nthreads
+);
+int_t fit_most_popular_internal
+(
+    real_t *restrict biasA, real_t *restrict biasB,
+    real_t *restrict glob_mean, bool center,
+    real_t lam_user, real_t lam_item,
+    bool scale_lam,
     real_t alpha,
     int_t m, int_t n,
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
@@ -839,13 +894,16 @@ void collective_closed_form_block
     real_t *restrict Xones, int_t incXones,
     real_t *restrict weight,
     real_t lam, real_t w_user, real_t w_implicit, real_t lam_last,
+    real_t l1_lam, real_t l1_lam_bias,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t *restrict precomputedBtB, int_t cnt_NA_x,
     real_t *restrict precomputedCtCw, int_t cnt_NA_u,
     real_t *restrict precomputedBeTBeChol, int_t n_BtB,
     real_t *restrict precomputedBiTBi,
     bool add_X, bool add_U,
     bool use_cg, int_t max_cg_steps,/* <- 'cg' should not be used for new data*/
-    bool nonneg, int_t max_cd_steps, real_t *restrict a_prev,
+    bool nonneg, int_t max_cd_steps,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     real_t *restrict buffer_real_t
 );
 void collective_closed_form_block_implicit
@@ -857,14 +915,14 @@ void collective_closed_form_block_implicit
     real_t *restrict u_vec, int_t cnt_NA_u,
     real_t *restrict u_vec_sp, int_t u_vec_ixB[], size_t nnz_u_vec,
     bool NA_as_zero_U,
-    real_t lam, real_t w_user,
+    real_t lam, real_t l1_lam, real_t w_user,
     real_t *restrict precomputedBeTBe,
     real_t *restrict precomputedBtB, /* for cg, should NOT have lambda added */
     real_t *restrict precomputedBeTBeChol,
     real_t *restrict precomputedCtCw,
     bool add_U, bool shapes_match,
     bool use_cg, int_t max_cg_steps,/* <- 'cg' should not be used for new data*/
-    bool nonneg, int_t max_cd_steps, real_t *restrict a_prev,
+    bool nonneg, int_t max_cd_steps,
     real_t *restrict buffer_real_t
 );
 void collective_block_cg
@@ -887,6 +945,7 @@ void collective_block_cg
     real_t *restrict precomputedBtB,
     real_t *restrict precomputedCtC, /* should NOT be multiplied by 'w_user' */
     int_t max_cg_steps,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     real_t *restrict buffer_real_t
 );
 void collective_block_cg_implicit
@@ -915,10 +974,10 @@ void optimizeA_collective_implicit
     size_t U_csr_p[], int_t U_csr_i[], real_t *restrict U_csr,
     real_t *restrict U, int_t cnt_NA_u[],
     bool full_dense_u, bool near_dense_u, bool NA_as_zero_U,
-    real_t lam, real_t w_user,
+    real_t lam, real_t l1_lam, real_t w_user,
     int_t nthreads,
     bool use_cg, int_t max_cg_steps, bool is_first_iter,
-    bool nonneg, int_t max_cd_steps, real_t *restrict A_prev,
+    bool nonneg, int_t max_cd_steps,
     real_t *restrict precomputedBtB, /* will not have lambda with CG */
     real_t *restrict precomputedBeTBe,
     real_t *restrict precomputedBeTBeChol,
@@ -939,7 +998,8 @@ int_t collective_factors_cold
     real_t *restrict CtCw,
     real_t *restrict col_means,
     int_t k, int_t k_user, int_t k_main,
-    real_t lam, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t w_main, real_t w_user,
+    bool scale_lam,
     bool NA_as_zero_U,
     bool nonneg
 );
@@ -955,7 +1015,8 @@ int_t collective_factors_cold_implicit
     real_t *restrict BeTBeChol,
     real_t *restrict col_means,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t w_main, real_t w_user, real_t w_main_multiplier,
+    real_t lam, real_t l1_lam,
+    real_t w_main, real_t w_user, real_t w_main_multiplier,
     bool NA_as_zero_U,
     bool nonneg
 );
@@ -975,8 +1036,11 @@ int_t collective_factors_warm
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t w_main, real_t w_user, real_t w_implicit,real_t lam_bias,
+    real_t l1_lam, real_t l1_lam_bias,
+    bool scale_lam, bool scale_lam_sideinfo,
     int_t n_max, bool include_all_X,
     real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BtB,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
@@ -996,7 +1060,7 @@ int_t collective_factors_warm_implicit
     real_t *restrict B, int_t n, real_t *restrict C,
     real_t *restrict Xa, int_t ixB[], size_t nnz,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t alpha, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
     real_t *restrict BeTBe,
     real_t *restrict BtB,
@@ -1051,7 +1115,7 @@ size_t buffer_size_optimizeA_collective
     bool add_implicit_features, size_t k_main_i,
     size_t nthreads,
     bool use_cg, bool finalize_chol,
-    bool nonneg,
+    bool nonneg, bool has_l1,
     bool keep_precomputed,
     bool pass_allocated_BtB,
     bool pass_allocated_CtCw,
@@ -1066,6 +1130,7 @@ size_t buffer_size_optimizeA_collective_implicit
     bool NA_as_zero_U,
     size_t nthreads,
     bool use_cg,
+    bool nonneg, bool has_l1,
     bool pass_allocated_BtB,
     bool pass_allocated_BeTBe,
     bool pass_allocated_BeTBeChol,
@@ -1088,16 +1153,20 @@ void optimizeA_collective
     real_t *restrict U, int_t cnt_NA_u[],
     bool full_dense_u, bool near_dense_u, bool NA_as_zero_U,
     real_t lam, real_t w_user, real_t w_implicit, real_t lam_last,
+    real_t l1_lam, real_t l1_lam_bias,
+    bool scale_lam, bool scale_lam_sideinfo,
     bool do_B,
     int_t nthreads,
     bool use_cg, int_t max_cg_steps, bool is_first_iter,
-    bool nonneg, int_t max_cd_steps, real_t *restrict A_prev,
+    bool nonneg, int_t max_cd_steps,
+    real_t *restrict bias_BtX, real_t *restrict bias_X, real_t bias_X_glob,
     bool keep_precomputed,
     real_t *restrict precomputedBtB,
     real_t *restrict precomputedCtCw,
     real_t *restrict precomputedBeTBeChol,
     real_t *restrict precomputedBiTBi,
     bool *filled_BtB, bool *filled_CtCw, bool *filled_BeTBeChol,
+    bool *CtC_is_scaled,
     real_t *restrict buffer_real_t
 );
 void build_BeTBe
@@ -1197,7 +1266,7 @@ int_t fit_collective_explicit_lbfgs_internal
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     real_t lam, real_t *restrict lam_unique,
     real_t *restrict U, int_t m_u, int_t p,
     real_t *restrict II, int_t n_i, int_t q,
@@ -1226,7 +1295,7 @@ int_t fit_collective_explicit_lbfgs
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     real_t lam, real_t *restrict lam_unique,
     real_t *restrict U, int_t m_u, int_t p,
     real_t *restrict II, int_t n_i, int_t q,
@@ -1263,8 +1332,10 @@ int_t fit_collective_explicit_als
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t *restrict U, int_t m_u, int_t p,
     real_t *restrict II, int_t n_i, int_t q,
     int_t U_row[], int_t U_col[], real_t *restrict U_sp, size_t nnz_U,
@@ -1280,6 +1351,7 @@ int_t fit_collective_explicit_als
     real_t *restrict B_plus_bias,
     real_t *restrict precomputedBtB,
     real_t *restrict precomputedTransBtBinvBt,
+    real_t *restrict precomputedBtXbias,
     real_t *restrict precomputedBeTBeChol,
     real_t *restrict precomputedBiTBi,
     real_t *restrict precomputedTransCtCinvCt,
@@ -1294,6 +1366,7 @@ int_t fit_collective_implicit_als
     int_t m, int_t n, int_t k,
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
     real_t *restrict U, int_t m_u, int_t p,
     real_t *restrict II, int_t n_i, int_t q,
     int_t U_row[], int_t U_col[], real_t *restrict U_sp, size_t nnz_U,
@@ -1316,14 +1389,17 @@ int_t precompute_collective_explicit
     real_t *restrict B, int_t n, int_t n_max, bool include_all_X,
     real_t *restrict C, int_t p,
     real_t *restrict Bi, bool add_implicit_features,
+    real_t *restrict biasB, real_t glob_mean, bool NA_as_zero_X,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     bool user_bias,
     bool nonneg,
     real_t lam, real_t *restrict lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     real_t *restrict B_plus_bias,
     real_t *restrict BtB,
     real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict TransCtCinvCt,
@@ -1359,10 +1435,13 @@ int_t factors_collective_explicit_single
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     int_t n_max, bool include_all_X,
-    real_t *restrict TransBtBinvBt,
     real_t *restrict BtB,
+    real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict CtCw,
@@ -1380,7 +1459,7 @@ int_t factors_collective_implicit_single
     real_t *restrict B, int_t n, real_t *restrict C,
     real_t *restrict Xa, int_t ixB[], size_t nnz,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t alpha, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
     bool apply_log_transf,
     real_t *restrict BeTBe,
@@ -1407,10 +1486,13 @@ int_t factors_collective_explicit_multiple
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     int_t n_max, bool include_all_X,
-    real_t *restrict TransBtBinvBt,
     real_t *restrict BtB,
+    real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict TransCtCinvCt,
@@ -1432,7 +1514,7 @@ int_t factors_collective_implicit_multiple
     real_t *restrict C,
     real_t *restrict U_colmeans,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t alpha, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
     bool apply_log_transf,
     real_t *restrict BeTBe,
@@ -1458,10 +1540,12 @@ int_t impute_X_collective_explicit
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     int_t n_max, bool include_all_X,
-    real_t *restrict TransBtBinvBt,
     real_t *restrict BtB,
+    real_t *restrict TransBtBinvBt,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict TransCtCinvCt,
@@ -1512,10 +1596,13 @@ int_t topN_new_collective_explicit
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     int_t n_max, bool include_all_X,
-    real_t *restrict TransBtBinvBt,
     real_t *restrict BtB,
+    real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict CtCw,
@@ -1535,11 +1622,11 @@ int_t topN_new_collective_implicit
     real_t *restrict u_vec_sp, int_t u_vec_ixB[], size_t nnz_u_vec,
     bool NA_as_zero_U,
     bool nonneg,
-    real_t *restrict col_means,
+    real_t *restrict U_colmeans,
     real_t *restrict B, real_t *restrict C,
     real_t *restrict Xa, int_t ixB[], size_t nnz,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t alpha, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
     bool apply_log_transf,
     real_t *restrict BeTBe,
@@ -1595,10 +1682,13 @@ int_t predict_X_new_collective_explicit
     real_t *restrict Bi, bool add_implicit_features,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t *restrict lam_unique,
+    real_t l1_lam, real_t *restrict l1_lam_unique,
+    bool scale_lam, bool scale_lam_sideinfo,
     real_t w_main, real_t w_user, real_t w_implicit,
     int_t n_max, bool include_all_X,
-    real_t *restrict TransBtBinvBt,
     real_t *restrict BtB,
+    real_t *restrict TransBtBinvBt,
+    real_t *restrict BtXbias,
     real_t *restrict BeTBeChol,
     real_t *restrict BiTBi,
     real_t *restrict TransCtCinvCt,
@@ -1621,9 +1711,9 @@ int_t predict_X_new_collective_implicit
     size_t *restrict Xcsr_p, int_t *restrict Xcsr_i, real_t *restrict Xcsr,
     real_t *restrict B, int_t n,
     real_t *restrict C,
-    real_t *restrict col_means,
+    real_t *restrict U_colmeans,
     int_t k, int_t k_user, int_t k_item, int_t k_main,
-    real_t lam, real_t alpha, real_t w_main, real_t w_user,
+    real_t lam, real_t l1_lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
     bool apply_log_transf,
     real_t *restrict BeTBe,
@@ -1712,6 +1802,7 @@ int_t precompute_offsets_both
     real_t *restrict C, int_t p,
     real_t *restrict D, int_t q,
     real_t *restrict C_bias, real_t *restrict D_bias,
+    real_t *restrict biasB, real_t glob_mean, bool NA_as_zero_X,
     bool user_bias, bool add_intercepts, bool implicit,
     int_t k, int_t k_main, int_t k_sec,
     real_t lam, real_t *restrict lam_unique,
@@ -1811,7 +1902,7 @@ int_t fit_offsets_explicit_lbfgs_internal
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     bool add_intercepts,
     real_t lam, real_t *restrict lam_unique,
     real_t *restrict U, int_t p,
@@ -1839,7 +1930,7 @@ int_t fit_offsets_explicit_lbfgs
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias,
+    bool user_bias, bool item_bias, bool center,
     bool add_intercepts,
     real_t lam, real_t *restrict lam_unique,
     real_t *restrict U, int_t p,
@@ -1870,7 +1961,7 @@ int_t fit_offsets_als
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias, bool add_intercepts,
+    bool user_bias, bool item_bias, bool center, bool add_intercepts,
     real_t lam,
     real_t *restrict U, int_t p,
     real_t *restrict II, int_t q,
@@ -1898,7 +1989,7 @@ int_t fit_offsets_explicit_als
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool user_bias, bool item_bias, bool add_intercepts,
+    bool user_bias, bool item_bias, bool center, bool add_intercepts,
     real_t lam,
     real_t *restrict U, int_t p,
     real_t *restrict II, int_t q,
